@@ -18,25 +18,10 @@ __docformat__ = 'restructuredtext'
 import re
 
 para_sep = re.compile('\n{2,}')
-whitespace=re.compile('[ \t\n\r]+')
+whitespace = re.compile('[ \t\n\r]+')
 
 def wrap(text, width=78, indent=0):
     """Makes sure that we keep a line length of a certain width.
-
-    Examples:
-
-    >>> print wrap('foo bar')[:-2]
-    foo bar
-    >>> print wrap('foo bar', indent=2)[:-2]
-      foo bar
-    >>> print wrap('foo bar, more foo bar', 10)[:-2]
-    foo bar,
-    more foo
-    bar
-    >>> print wrap('foo bar, more foo bar', 10, 2)[:-2]
-      foo bar,
-      more foo
-      bar
     """
     paras = para_sep.split(text.strip())
 
