@@ -16,46 +16,48 @@
 Note, for a detailed description of the way that conflicting
 configuration actions are resolved, see the detailed example in
 test_includeOverrides in tests/test_xmlconfig.py
-
-$Id: xmlconfig.py 110550 2010-04-06 06:50:36Z tseaver $
 """
 __docformat__ = 'restructuredtext'
 
 import errno
+from glob import glob
+import logging
 import os
 import sys
-import logging
-import zope.configuration.config as config
-
-from glob import glob
 from xml.sax import make_parser
 from xml.sax.xmlreader import InputSource
 from xml.sax.handler import ContentHandler, feature_namespaces
 from xml.sax import SAXParseException
-from zope import schema
-from zope.configuration.exceptions import ConfigurationError
-from zope.configuration.zopeconfigure import IZopeConfigure, ZopeConfigure
+
 from zope.interface import Interface
+from zope.schema import NativeStringLine
+
+from zope.configuration.config import ConfigurationMachine
+from zope.configuration.config import defineGroupingDirective
+from zope.configuration.config import defineSimpleDirective
+from zope.configuration.config import GroupingContextDecorator
+from zope.configuration.config import GroupingStackItem
+from zope.configuration.config import resolveConflicts
+from zope.configuration.exceptions import ConfigurationError
+from zope.configuration.fields import GlobalObject
+from zope.configuration.zopeconfigure import IZopeConfigure
+from zope.configuration.zopeconfigure import ZopeConfigure
+from zope.configuration._compat import StringIO
+from zope.configuration._compat import reraise
+from zope.configuration._compat import u
 
 logger = logging.getLogger("config")
 
 ZCML_NAMESPACE = "http://namespaces.zope.org/zcml"
-ZCML_CONDITION = (ZCML_NAMESPACE, u"condition")
+ZCML_CONDITION = (ZCML_NAMESPACE, u("condition"))
 
 
 class ZopeXMLConfigurationError(ConfigurationError):
     """Zope XML Configuration error
 
-    These errors are wrappers for other errors. The include configuration
-    info and the wrapped error type and value:
-
-    >>> v = ZopeXMLConfigurationError("blah", AttributeError, "xxx")
-    >>> print v
-    'blah'
-        AttributeError: xxx
-
+    These errors are wrappers for other errors. They include configuration
+    info and the wrapped error type and value.
     """
-
     def __init__(self, info, etype, evalue):
         self.info, self.etype, self.evalue = info, etype, evalue
 
@@ -63,17 +65,11 @@ class ZopeXMLConfigurationError(ConfigurationError):
         # Only use the repr of the info. This is because we expect to
         # get a parse info and we only want the location information.
         return "%s\n    %s: %s" % (
-            `self.info`, self.etype.__name__, self.evalue)
+            repr(self.info), self.etype.__name__, self.evalue)
 
 class ZopeSAXParseException(ConfigurationError):
     """Sax Parser errors, reformatted in an emacs friendly way
-
-    >>> v = ZopeSAXParseException("foo.xml:12:3:Not well formed")
-    >>> print v
-    File "foo.xml", line 12.3, Not well formed
-
     """
-
     def __init__(self, v):
         self._v = v
 
@@ -90,37 +86,8 @@ class ParserInfo(object):
 
     This includes the directive location, as well as text data
     contained in the directive.
-
-    >>> info = ParserInfo('tests//sample.zcml', 1, 0)
-    >>> info
-    File "tests//sample.zcml", line 1.0
-
-    >>> print info
-    File "tests//sample.zcml", line 1.0
-
-    >>> info.characters("blah\\n")
-    >>> info.characters("blah")
-    >>> info.text
-    u'blah\\nblah'
-
-    >>> info.end(7, 0)
-    >>> info
-    File "tests//sample.zcml", line 1.0-7.0
-
-    >>> print info
-    File "tests//sample.zcml", line 1.0-7.0
-      <configure xmlns='http://namespaces.zope.org/zope'>
-        <!-- zope.configure -->
-        <directives namespace="http://namespaces.zope.org/zope">
-          <directive name="hook" attributes="name implementation module"
-             handler="zope.configuration.metaconfigure.hook" />
-        </directives>
-      </configure>
-
-
     """
-
-    text = u''
+    text = u('')
 
     def __init__(self, file, line, column):
         self.file, self.line, self.column = file, line, column
@@ -149,37 +116,39 @@ class ParserInfo(object):
                                 'tests', 'sample.zcml')
 
         try:
-            f = open(file)
+            with open(file) as f:
+                lines = f.readlines()[self.line-1:self.eline]
         except IOError:
             src = "  Could not read source."
         else:
-            lines = f.readlines()[self.line-1:self.eline]
             ecolumn = self.ecolumn
-            if lines[-1][ecolumn:ecolumn+2] == '</':
+            if lines[-1][ecolumn:ecolumn+2] == '</': #pragma NO COVER
                 # We're pointing to the start of an end tag. Try to find
                 # the end
                 l = lines[-1].find('>', ecolumn)
                 if l >= 0:
                     lines[-1] = lines[-1][:l+1]
-            else:
+            else: #pragma NO COVER
                 lines[-1] = lines[-1][:ecolumn+1]
 
             column = self.column
-            if lines[0][:column].strip():
+            if lines[0][:column].strip(): #pragma NO COVER
                 # Remove text before start if it's noy whitespace
                 lines[0] = lines[0][self.column:]
 
+            pad = u('  ')
+            blank = u('')
             try:
-                src = u''.join([u"  "+l for l in lines])
-            except UnicodeDecodeError:
+                src = blank.join([pad + l for l in lines])
+            except UnicodeDecodeError: #pragma NO COVER
                 # XXX:
                 # I hope so most internation zcml will use UTF-8 as encoding
                 # otherwise this code must be made more clever
-                src = u''.join([u"  "+l.decode('utf-8') for l in lines])
+                src = blank.join([pad + l.decode('utf-8') for l in lines])
                 # unicode won't be printable, at least on my console
                 src = src.encode('ascii','replace')
 
-        return "%s\n%s" % (`self`, src)
+        return "%s\n%s" % (repr(self), src)
 
     def characters(self, characters):
         self.text += characters
@@ -190,8 +159,9 @@ class ConfigurationHandler(ContentHandler):
 
     Translate parser events into calls into the configuration system.
     """
+    locator = None
 
-    def __init__(self, context, testing=0):
+    def __init__(self, context, testing=False):
         self.context = context
         self.testing = testing
         self.ignore_depth = 0
@@ -232,84 +202,25 @@ class ConfigurationHandler(ContentHandler):
 
         try:
             self.context.begin(name, data, info)
-        except (KeyboardInterrupt, SystemExit):
+        except (KeyboardInterrupt, SystemExit): #pragma NO COVER
             raise
         except:
             if self.testing:
                 raise
-            raise ZopeXMLConfigurationError(info, sys.exc_info()[0],
-                sys.exc_info()[1]), None, sys.exc_info()[2]
+            reraise(ZopeXMLConfigurationError(info,
+                                              sys.exc_info()[0],
+                                              sys.exc_info()[1]),
+                    None, sys.exc_info()[2])
 
         self.context.setInfo(info)
 
     def evaluateCondition(self, expression):
         """Evaluate a ZCML condition.
 
-        `expression` is a string of the form "verb arguments".
+        ``expression`` is a string of the form "verb arguments".
 
-        Currently the supported verbs are 'have', 'not-have',
-        'installed' and 'not-installed'.
-
-        The 'have' verb takes one argument: the name of a feature.
-
-        >>> from zope.configuration.config import ConfigurationContext
-        >>> context = ConfigurationContext()
-        >>> context.provideFeature('apidoc')
-        >>> c = ConfigurationHandler(context, testing=True)
-        >>> c.evaluateCondition("have apidoc")
-        True
-        >>> c.evaluateCondition("not-have apidoc")
-        False
-        >>> c.evaluateCondition("have onlinehelp")
-        False
-        >>> c.evaluateCondition("not-have onlinehelp")
-        True
-
-        Ill-formed expressions raise an error
-
-        >>> c.evaluateCondition("want apidoc")
-        Traceback (most recent call last):
-          ...
-        ValueError: Invalid ZCML condition: 'want apidoc'
-
-        >>> c.evaluateCondition("have x y")
-        Traceback (most recent call last):
-          ...
-        ValueError: Only one feature allowed: 'have x y'
-
-        >>> c.evaluateCondition("have")
-        Traceback (most recent call last):
-          ...
-        ValueError: Feature name missing: 'have'
-
-
-        The 'installed' verb takes one argument: the dotted name of a
-        pacakge. If the pacakge is found, in other words, can be imported,
-        then the condition will return true.
-
-        >>> from zope.configuration.config import ConfigurationContext
-        >>> context = ConfigurationContext()
-        >>> c = ConfigurationHandler(context, testing=True)
-        >>> c.evaluateCondition('installed zope.interface')
-        True
-        >>> c.evaluateCondition('not-installed zope.interface')
-        False
-        >>> c.evaluateCondition('installed zope.foo')
-        False
-        >>> c.evaluateCondition('not-installed zope.foo')
-        True
-
-        Ill-formed expressions raise an error
-
-        >>> c.evaluateCondition("installed foo bar")
-        Traceback (most recent call last):
-          ...
-        ValueError: Only one package allowed: 'installed foo bar'
-
-        >>> c.evaluateCondition("installed")
-        Traceback (most recent call last):
-          ...
-        ValueError: Package name missing: 'installed'
+        Currently the supported verbs are ``have``, ``not-have``,
+        ``installed`` and ``not-installed``.
         """
         arguments = expression.split(None)
         verb = arguments.pop(0)
@@ -359,13 +270,15 @@ class ConfigurationHandler(ContentHandler):
 
         try:
             self.context.end()
-        except (KeyboardInterrupt, SystemExit):
+        except (KeyboardInterrupt, SystemExit): #pragma NO COVER
             raise
         except:
             if self.testing:
                 raise
-            raise ZopeXMLConfigurationError(info, sys.exc_info()[0],
-                sys.exc_info()[1]), None, sys.exc_info()[2]
+            reraise(ZopeXMLConfigurationError(info,
+                                              sys.exc_info()[0],
+                                              sys.exc_info()[1]),
+                    None, sys.exc_info()[2])
 
 
 def processxmlfile(file, context, testing=False):
@@ -381,7 +294,8 @@ def processxmlfile(file, context, testing=False):
     try:
         parser.parse(src)
     except SAXParseException:
-        raise ZopeSAXParseException(sys.exc_info()[1]), None, sys.exc_info()[2]
+        reraise(ZopeSAXParseException(sys.exc_info()[1]),
+                None, sys.exc_info()[2])
 
 
 def openInOrPlain(filename):
@@ -390,55 +304,17 @@ def openInOrPlain(filename):
     If the requested file does not exist and filename.in does, fall
     back to filename.in.  If opening the original filename fails for
     any other reason, allow the failure to propogate.
-
-    For example, the tests/samplepackage dirextory has files:
-
-       configure.zcml
-       configure.zcml.in
-       foo.zcml.in
-
-    If we open configure.zcml, we'll get that file:
-
-    >>> here = os.path.dirname(__file__)
-    >>> path = os.path.join(here, 'tests', 'samplepackage', 'configure.zcml')
-    >>> f = openInOrPlain(path)
-    >>> f.name[-14:]
-    'configure.zcml'
-
-    But if we open foo.zcml, we'll get foo.zcml.in, since there isn't a
-    foo.zcml:
-
-    >>> path = os.path.join(here, 'tests', 'samplepackage', 'foo.zcml')
-    >>> f = openInOrPlain(path)
-    >>> f.name[-11:]
-    'foo.zcml.in'
-
-    Make sure other IOErrors are re-raised.  We need to do this in a
-    try-except block because different errors are raised on Windows and
-    on Linux.
-
-    >>> try:
-    ...     f = openInOrPlain('.')
-    ... except IOError:
-    ...     print "passed"
-    ... else:
-    ...     print "failed"
-    ...
-    passed
-
     """
     try:
-        fp = open(filename)
-    except IOError, (code, msg):
+        return open(filename)
+    except IOError as e:
+        code, msg = e.args
         if code == errno.ENOENT:
             fn = filename + ".in"
             if os.path.exists(fn):
-                fp = open(fn)
-            else:
-                raise
-        else:
-            raise
-    return fp
+                return open(fn)
+        raise
+
 
 class IInclude(Interface):
     """The ``include``, ``includeOverrides`` and ``exclude`` directives
@@ -448,21 +324,21 @@ class IInclude(Interface):
     files in each package and then link them together.
     """
 
-    file = schema.BytesLine(
-        title=u"Configuration file name",
-        description=u"The name of a configuration file to be included/excluded, "
-                    u"relative to the directive containing the "
-                    u"including configuration file.",
+    file = NativeStringLine(
+        title=u("Configuration file name"),
+        description=u("The name of a configuration file to be included/"
+                      "excluded, relative to the directive containing the "
+                      "including configuration file."),
         required=False,
         )
 
-    files = schema.BytesLine(
-        title=u"Configuration file name pattern",
-        description=u"""
+    files = NativeStringLine(
+        title=u("Configuration file name pattern"),
+        description=u("""
         The names of multiple configuration files to be included/excluded,
         expressed as a file-name pattern, relative to the directive
-        containing the including or excluding configuration file.  The pattern
-        can include:
+        containing the including or excluding configuration file.
+        The pattern can include:
 
         - ``*`` matches 0 or more characters
 
@@ -474,16 +350,16 @@ class IInclude(Interface):
 
         The file names are included in sorted order, where sorting is
         without regard to case.
-        """,
+        """),
         required=False,
         )
 
-    package = config.fields.GlobalObject(
-        title=u"Include or exclude package",
-        description=u"""
-        Include or exclude the named file (or configure.zcml) from the directory
-        of this package.
-        """,
+    package = GlobalObject(
+        title=u("Include or exclude package"),
+        description=u("""
+        Include or exclude the named file (or configure.zcml) from the
+        directory of this package.
+        """),
         required=False,
         )
 
@@ -500,55 +376,30 @@ def include(_context, file=None, package=None, files=None):
     elif not file:
         file = 'configure.zcml'
 
-    # BBB 2006/12/19 -- to be removed after 12 months
-    # This is a backward-compatibility support for old site.conf
-
-    if package and (package.__name__ == 'zope.app'):
-        try:
-            import zope.app.zcmlfiles
-        except ImportError:
-            pass # maybe this is an old zope without zope.app.zcmlfiles
-        else:
-            dirpath, filename = os.path.split(file)
-            # be careful, because zope.app is a namespace package
-            # we can't assume that zcmlfiles is a subdirectory of the 
-            # zope.app package
-            dirpath = os.path.dirname(zope.app.zcmlfiles.__file__)
-            file = os.path.join(dirpath, filename)
-            import warnings
-            warnings.warn('In configuration file: %s '
-                          'replace: <include package="zope.app" /> '
-                          'with: <include package="zope.app.zcmlfiles" /> '
-                          'This will go away in Zope 3.6.' % os.path.abspath(file),
-                          DeprecationWarning,
-                          2)
-
     # This is a tad tricky. We want to behave as a grouping directive.
 
-    context = config.GroupingContextDecorator(_context)
+    context = GroupingContextDecorator(_context)
     if package is not None:
         context.package = package
         context.basepath = None
 
     if files:
         paths = glob(context.path(files))
-        paths = zip([path.lower() for path in paths], paths)
-        paths.sort()
+        paths = sorted(zip([path.lower() for path in paths], paths))
         paths = [path for (l, path) in paths]
     else:
         paths = [context.path(file)]
 
     for path in paths:
         if context.processFile(path):
-            f = openInOrPlain(path)
-            logger.debug("include %s" % f.name)
+            with openInOrPlain(path) as f:
+                logger.debug("include %s" % f.name)
 
-            context.basepath = os.path.dirname(path)
-            context.includepath = _context.includepath + (f.name, )
-            _context.stack.append(config.GroupingStackItem(context))
+                context.basepath = os.path.dirname(path)
+                context.includepath = _context.includepath + (f.name, )
+                _context.stack.append(GroupingStackItem(context))
 
-            processxmlfile(f, context)
-            f.close()
+                processxmlfile(f, context)
             assert _context.stack[-1].context is context
             _context.stack.pop()
 
@@ -566,15 +417,14 @@ def exclude(_context, file=None, package=None, files=None):
         file = 'configure.zcml'
 
 
-    context = config.GroupingContextDecorator(_context)
+    context = GroupingContextDecorator(_context)
     if package is not None:
         context.package = package
         context.basepath = None
 
     if files:
         paths = glob(context.path(files))
-        paths = zip([path.lower() for path in paths], paths)
-        paths.sort()
+        paths = sorted(zip([path.lower() for path in paths], paths))
         paths = [path for (l, path) in paths]
     else:
         paths = [context.path(file)]
@@ -608,30 +458,27 @@ def includeOverrides(_context, file=None, package=None, files=None):
     # Now we'll grab the new actions, resolve conflicts,
     # and munge the includepath:
     newactions = []
-    for action in config.resolveConflicts(_context.actions[nactions:]):
-        (discriminator, callable, args, kw, oldincludepath, info, order
-         ) = config.expand_action(*action)
-        newactions.append(
-            (discriminator, callable, args, kw, includepath, info, order)
-            )
 
-    # and replace the new actions with the munched new actions:
+    for action in resolveConflicts(_context.actions[nactions:]):
+        action['includepath'] = includepath
+        newactions.append(action)
+
     _context.actions[nactions:] = newactions
 
 def registerCommonDirectives(context):
     # We have to use the direct definition functions to define
     # a directive for all namespaces.
 
-    config.defineSimpleDirective(
+    defineSimpleDirective(
         context, "include", IInclude, include, namespace="*")
 
-    config.defineSimpleDirective(
+    defineSimpleDirective(
         context, "exclude", IInclude, exclude, namespace="*")
 
-    config.defineSimpleDirective(
+    defineSimpleDirective(
         context, "includeOverrides", IInclude, includeOverrides, namespace="*")
 
-    config.defineGroupingDirective(
+    defineGroupingDirective(
         context,
         name="configure",
         namespace="*",
@@ -644,7 +491,7 @@ def file(name, package=None, context=None, execute=True):
     """
 
     if context is None:
-        context = config.ConfigurationMachine()
+        context = ConfigurationMachine()
         registerCommonDirectives(context)
         context.package = package
 
@@ -657,10 +504,8 @@ def file(name, package=None, context=None, execute=True):
 def string(s, context=None, name="<string>", execute=True):
     """Execute a zcml string
     """
-    from StringIO import StringIO
-
     if context is None:
-        context = config.ConfigurationMachine()
+        context = ConfigurationMachine()
         registerCommonDirectives(context)
 
     f = StringIO(s)
@@ -680,7 +525,7 @@ def string(s, context=None, name="<string>", execute=True):
 _context = None
 def _clearContext():
     global _context
-    _context = config.ConfigurationMachine()
+    _context = ConfigurationMachine()
     registerCommonDirectives(_context)
 
 def _getContext():
@@ -689,9 +534,9 @@ def _getContext():
         _clearContext()
         try:
             from zope.testing.cleanup import addCleanUp
-        except ImportError:
+        except ImportError: #pragma NO COVER
             pass
-        else:
+        else: #pragma NO COVER
             addCleanUp(_clearContext)
             del addCleanUp
     return _context
@@ -716,7 +561,7 @@ def xmlconfig(file, testing=False):
     context.execute_actions(testing=testing)
 
 
-def testxmlconfig(file, context=None):
+def testxmlconfig(file):
     """xmlconfig that doesn't raise configuration errors
 
     This is useful for testing, as it doesn't mask exception types.
